@@ -1,7 +1,8 @@
+from .beams import *
 from .calibration import *
 from .operators import *
 from .instrument import *
 from .acquisition import *
 from .pointings import *
 
-__version__ = '2.5'
+__version__ = '2.5.0'
